/*
 * Copyright (c) 2019 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr/zephyr.h>
#include <nrfx_gpiote.h>
#include <helpers/nrfx_gppi.h>
#include <hal/nrf_rtc.h>
#include <nrfx_rtc.h>
#include <nrfx_ppi.h>
#include <zephyr/logging/log.h>

LOG_MODULE_REGISTER(nrfx_rtc_capture_clear_ppi_sample, LOG_LEVEL_INF);

#define OUTPUT_PIN	DT_GPIO_PIN(DT_ALIAS(led0), gpios)

static nrfx_rtc_t rtc = NRFX_RTC_INSTANCE(2);

static const nrfx_rtc_config_t rtc_conf = {
    .prescaler          = RTC_FREQ_TO_PRESCALER(32768),
    .interrupt_priority = NRFX_RTC_DEFAULT_CONFIG_IRQ_PRIORITY,
    .tick_latency       = 0,
    .reliable           = false,
};


static void dummy_rtc_handler(nrfx_rtc_int_type_t int_type)
{
    LOG_INF("RTC interrupt is not needed. Should never see this log message.");
}


void rtc_init()
{
	nrfx_err_t err;

}


void main(void)
{
	LOG_INF("nrfx_rtc_capture_clear_ppi sample on %s", CONFIG_BOARD);

	nrfx_err_t err;
	uint8_t in_channel, out_channel;
	uint8_t ppi_channel;

	/* Initialize the RTC */
	err = nrfx_rtc_init(&rtc, &rtc_conf, dummy_rtc_handler);
	nrfx_rtc_tick_disable(&rtc);
	nrfx_rtc_overflow_disable(&rtc);
	err = nrfx_rtc_cc_set(&rtc, 0, 32768, false);
	nrfx_rtc_enable(&rtc);

	/* Initialize GPIOTE (the interrupt priority passed as the parameter
	 * here is ignored, see nrfx_glue.h).
	 */
	err = nrfx_gpiote_init(0);
	if (err != NRFX_SUCCESS) {
		LOG_ERR("nrfx_gpiote_init error: 0x%08X", err);
		return;
	}

	err = nrfx_gpiote_channel_alloc(&in_channel);
	if (err != NRFX_SUCCESS) {
		LOG_ERR("Failed to allocate in_channel, error: 0x%08X", err);
		return;
	}

	err = nrfx_gpiote_channel_alloc(&out_channel);
	if (err != NRFX_SUCCESS) {
		LOG_ERR("Failed to allocate out_channel, error: 0x%08X", err);
		return;
	}

	/* Initialize output pin. SET task will turn the LED on,
	 * CLR will turn it off and OUT will toggle it.
	 */
	static const nrfx_gpiote_output_config_t output_config = {
		.drive = NRF_GPIO_PIN_S0S1,
		.input_connect = NRF_GPIO_PIN_INPUT_DISCONNECT,
		.pull = NRF_GPIO_PIN_NOPULL,
	};
	const nrfx_gpiote_task_config_t task_config = {
		.task_ch = out_channel,
		.polarity = NRF_GPIOTE_POLARITY_TOGGLE,
		.init_val = 1,
	};
	err = nrfx_gpiote_output_configure(OUTPUT_PIN,
					   &output_config,
					   &task_config);
	if (err != NRFX_SUCCESS) {
		LOG_ERR("nrfx_gpiote_output_configure error: 0x%08X", err);
		return;
	}

	nrfx_gpiote_out_task_enable(OUTPUT_PIN);

	LOG_INF("nrfx_gpiote initialized");

	/* Allocate a PPI channel. */
	err = nrfx_gppi_channel_alloc(&ppi_channel);
	if (err != NRFX_SUCCESS) {
		LOG_ERR("nrfx_gppi_channel_alloc error: 0x%08X", err);
		return;
	}

	/* Configure endpoints of the channel so that the input pin event is
	 * connected with the output pin OUT task. This means that each time
	 * the button is pressed, the LED pin will be toggled.
	 */
	nrfx_gppi_channel_endpoints_setup(ppi_channel,
		nrfx_rtc_event_address_get(&rtc, NRF_RTC_EVENT_COMPARE_0),
		nrfx_gpiote_out_task_addr_get(OUTPUT_PIN));

	/* Clear the RTC on RTC via PPI on the compare 0 event.Using fork mechanism here.
	 * devices without fork mechanism (nRF53, nRF91)
	 */
	nrfx_gppi_fork_endpoint_setup(ppi_channel, nrfx_rtc_task_address_get(&rtc, NRF_RTC_TASK_CLEAR));

	/* Enable the channel. */
	nrfx_gppi_channels_enable(BIT(ppi_channel));

	LOG_INF("PPI configured, leaving main()");
}
